/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             Interface to ListX
*  SCOPE :              Interface to ListX
*  FILE :               listx.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma interface
#endif
#ifndef __listx_HXX
#define __listx_HXX

/* IMPORTS *******************************************************************/

#ifndef __colx_HXX
#include "colx.hxx"
#endif

#ifndef __stringx_HXX
#include "stringx.hxx"
#endif

/*****************************************************************************/

class LocalGenListX : public LocalColX {

protected :

  // Return pointer on nth element of the collection
  void* getElem(int ind) ;

public :
  // Initialise (if needed) a new collection
  void init(int s = YOO_GROWING_STEP) ;

  // Remove an element from collection
  void remove(int pos) ;
} ;


class LocalListXObjectX : public LocalGenListX {

protected :

  // Test membership of an element. Return position of the element or -1.
  int member(const yoo_ObjectX&) ;

  // Copy parameter into receiver
  void assign(const LocalListXObjectX&) ;

  // Append elements of the parameter at the end of the receiver
  void append(const LocalListXObjectX&) ;

  // Append the element at the end of the receiver
  void append(const yoo_ObjectX& el) {  this->insert(el, COL_TAIL) ; }

  // Compare elements (pointer) from receiver and parameter
  int compare(const LocalListXObjectX&) ;

  // Insert an element at position 'pos'
  void insert(const yoo_ObjectX& el, int pos) ;
} ;


#include "listx_tmpl.hxx"


class LocalListX<int> : public LocalGenListX {

  public :

  LocalListX() {} ;
  LocalListX(const LocalListX<int>& l) { *this = l ; } 

  void insert(int el, int pos) {
    this->init() ;
    char transfert[sizeof(ColElem)] ;
    this->domainI(pos) ; 
    memcpy(transfert, &el, sizeof(int)) ;
    this->_col = this->_col->insert(transfert, pos) ;
  }

  void assign(const LocalListX<int>& s) ;
  
  LocalListX<int>& operator=(const LocalListX<int>& s) {
    this->assign(s) ; return *this ;
  }

  LocalListX<int>& operator+=(const LocalListX<int>& s) ;

  LocalListX<int>& operator+=(int el) {
    this->insert(el, COL_TAIL) ;
    return *this ;
  }

  int operator==(const LocalListX<int>&) ;

  int operator!=(const LocalListX<int>& l) {
    return ! (*this == l) ; 
  }

  int member(int) ;

  int& operator[](int ind) {
    return *(int*)this->getElem(ind) ;
  }
} ;


class LocalListX<double> : public LocalGenListX {

  public :

  void insert(double el, int pos) {
    this->init() ;
    char transfert[YOO_SIZEOF_LIST_ELEM] ;
    this->domainI(pos) ; 
    memcpy(transfert, &el, sizeof(double)) ;
    this->_col = this->_col->insert(transfert, pos) ;
  }

  LocalListX() {} ;
  LocalListX(const LocalListX<double>& l) { *this = l ; } 

  void assign(const LocalListX<double>& s) ;

  LocalListX<double>& operator=(const LocalListX<double>& s) {
    this->assign(s) ; return *this ;
  }

  LocalListX<double>& operator+=(const LocalListX<double>& s) ;

  LocalListX<double>& operator+=(double el) {
    this->insert(el, COL_TAIL) ;
    return *this ;
  }

  int operator==(const LocalListX<double>&) ;

  int operator!=(const LocalListX<double>& l) {
    return ! (*this == l) ; 
  }

  int member(double) ;

  double& operator[](int ind) {
    return *(double*)this->getElem(ind) ;
  }
} ;


class LocalListX<StringX> : public LocalGenListX {

  public :

  LocalListX() {} ;

  LocalListX(const LocalListX<StringX>& l) { *this = l ; } 

  ~LocalListX() { this->destroy() ; }

  void assign(const LocalListX<StringX>& s) ;

  LocalListX<StringX>& operator=(const LocalListX<StringX>& s) {
    this->assign(s) ; return *this ;
  }

  LocalListX<StringX>& operator+=(const LocalListX<StringX>& s) ;

  LocalListX<StringX>& operator+=(char* el) {
    this->insert(el, COL_TAIL) ;
    return *this ;
  }

  int operator==(const LocalListX<StringX>&) ;

  int operator!=(const LocalListX<StringX>& l) {
    return ! (*this == l) ; 
  }

  int member(const StringX&) ;

  void clear() ;

  void insert(char* el, int pos) ;

  StringX& operator[](int ind) {
    return *(StringX*)this->getElem(ind) ;
  }

  private :

  void destroy() ;
} ;

yoo_declare(ListX<int>) ;
yoo_declare(ListX<double>) ;
yoo_declare(ListX<StringX>) ;
yoo_declare(ListX<ObjectX>) ;

#define yoo_declare_ListX(ELEM) \
     template class Iterator<ELEM> ; \
     template class LocalListX<ELEM> ; \
     template class ListX<ELEM> ; \
     template class Ref<ListX<ELEM> > ;


#endif
